@@ -11,6 +11,7 @@
 
 import time
 import hashlib
+from datetime import datetime
 
 try:
     from lxml import etree
@@ -21,7 +22,7 @@ except ImportError:
 
 
 __all__ = ('Weixin',)
-__version__ = '0.2.0'
+__version__ = '0.3.0'
 __author__ = 'Hsiaoming Yang <me@lepture.com>'
 
 
@@ -34,6 +35,7 @@ class Weixin(object):
     def __init__(self, app=None):
         self.token = None
         self._registry = {}
+        self._registry_without_key = []
 
         if app:
             self.init_app(app)
@@ -85,40 +87,67 @@ class Weixin(object):
 
         :param content: A text of xml body.
         """
-        dct = {}
+        raw = {}
         root = etree.fromstring(content)
         for child in root:
-            dct[child.tag] = child.text
+            raw[child.tag] = child.text
 
-        ret = {}
-        ret['id'] = dct.get('MsgId')
-        ret['timestamp'] = int(dct.get('CreateTime', 0))
-        ret['receiver'] = dct.get('ToUserName')
-        ret['sender'] = dct.get('FromUserName')
-        ret['type'] = type = dct.get('MsgType')
+        formatted = self.format(raw)
 
-        if type == 'text':
-            ret['content'] = dct.get('Content')
-            return ret
+        msg_type = formatted['type']
+        msg_parser = getattr(self, 'parse_%s' % msg_type, None)
+        if callable(msg_parser):
+            parsed = msg_parser(raw)
+        else:
+            parsed = self.parse_invalid_type(raw)
 
-        if type == 'image':
-            ret['picurl'] = dct.get('PicUrl')
-            return ret
+        formatted.update(parsed)
+        return formatted
 
-        if type == 'location':
-            ret['location_x'] = dct.get('Location_X')
-            ret['location_y'] = dct.get('Location_Y')
-            ret['scale'] = int(dct.get('Scale', 0))
-            ret['label'] = dct.get('Label')
-            return ret
+    def format(self, kwargs):
+        timestamp = int(kwargs.get('CreateTime', 0))
+        return {
+            'id': kwargs.get('MsgId'),
+            'timestamp': timestamp,
+            'receiver': kwargs.get('ToUserName'),
+            'sender': kwargs.get('FromUserName'),
+            'type': kwargs.get('MsgType'),
+            'time': datetime.fromtimestamp(timestamp),
+        }
 
-        if type == 'link':
-            ret['title'] = dct.get('Title')
-            ret['description'] = dct.get('Description')
-            ret['url'] = dct.get('url')
-            return ret
+    def parse_text(self, raw):
+        return {'content': raw.get('Content')}
 
-        return ret
+    def parse_image(self, raw):
+        return {'picurl': raw.get('PicUrl')}
+
+    def parse_location(self, raw):
+        return {
+            'location_x': raw.get('Location_X'),
+            'location_y': raw.get('Location_Y'),
+            'scale': int(raw.get('Scale', 0)),
+            'label': raw.get('Label'),
+        }
+
+    def parse_link(self, raw):
+        return {
+            'title': raw.get('Title'),
+            'description': raw.get('Description'),
+            'url': raw.get('url'),
+        }
+
+    def parse_event(self, raw):
+        return {
+            'event': raw.get('Event'),
+            'event_key': raw.get('EventKey'),
+            'ticket': raw.get('Ticket'),
+            'latitude': raw.get('Latitude'),
+            'longitude': raw.get('Longitude'),
+            'precision': raw.get('Precision'),
+        }
+
+    def parse_invalid_type(self, raw):
+        return {}
 
     def reply(self, username, type='text', sender=None, **kwargs):
         """Create the reply text for weixin.
@@ -170,7 +199,7 @@ class Weixin(object):
 
         return None
 
-    def register(self, key, func=None):
+    def register(self, key=None, func=None, **kwargs):
         """Register a command helper function.
 
         You can register the function::
@@ -195,12 +224,16 @@ class Weixin(object):
                 )
         """
         if func:
-            self._registry[key] = func
-            return
+            if key is None:
+                limitation = frozenset(kwargs.items())
+                self._registry_without_key.append((func, limitation))
+            else:
+                self._registry[key] = func
+            return func
 
-        return self.__call__(key)
+        return self.__call__(key, **kwargs)
 
-    def __call__(self, key):
+    def __call__(self, key, **kwargs):
         """Register a reply function.
 
         Only available as a decorator::
@@ -214,7 +247,8 @@ class Weixin(object):
                 )
         """
         def wrapper(func):
-            self._registry[key] = func
+            self.register(key, func=func, **kwargs)
+            return func
 
         return wrapper
 
@@ -250,10 +284,18 @@ class Weixin(object):
 
         if ret['type'] == 'text' and ret['content'] in self._registry:
             func = self._registry[ret['content']]
-        elif '*' in self._registry:
-            func = self._registry['*']
         else:
-            func = 'failed'
+            ret_set = frozenset(ret.items())
+            matched_rules = (
+                _func for _func, _limitation in self._registry_without_key
+                if _limitation.issubset(ret_set))
+            func = next(matched_rules, None)  # first matched rule
+
+        if func is None:
+            if '*' in self._registry:
+                func = self._registry['*']
+            else:
+                func = 'failed'
 
         if callable(func):
             text = func(**ret)
@@ -264,6 +306,7 @@ class Weixin(object):
                 sender=ret['receiver'],
                 content=func,
             )
+
         return Response(text, content_type='text/xml; charset=utf-8')
 
     view_func.methods = ['GET', 'POST']
